from django.test import TestCase
from django.test.client import RequestFactory
from wagtail.models import Page

from tests.testapp.models import QRCodePage
from wagtail_qrcode.views import qr_code_page_view


class TestViews(TestCase):
    @classmethod
    def setUpTestData(self):
        self.request_factory = RequestFactory()
        self.root_page = Page.objects.get(id=1)

        home_page = self.root_page.get_children().first()

        self.test_qrcode_page = QRCodePage(title="Test Page")
        home_page.add_child(instance=self.test_qrcode_page)
        rev = self.test_qrcode_page.save_revision()
        rev.publish()

        self.test_wagtail_page = Page(title="Test Page")
        home_page.add_child(instance=self.test_wagtail_page)
        rev = self.test_wagtail_page.save_revision()
        rev.publish()

    def test_qr_code_view_404(self):
        request = self.request_factory.get("?id=0")
        response = qr_code_page_view(request)

        self.assertContains(response, "Page not found", status_code=404)

    def test_qr_code_view_qrcode_page(self):
        request = self.request_factory.get("?id={}".format(self.test_qrcode_page.id))
        response = qr_code_page_view(request)

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, self.test_qrcode_page.url)

    def test_qr_code_view_not_qrcode_page(self):
        request = self.request_factory.get("?id={}".format(self.test_wagtail_page.id))
        response = qr_code_page_view(request)

        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.content, b"Page not found")

    def test_qr_code_view_usage_count(self):
        request = self.request_factory.get(f"?id={self.test_qrcode_page.id}")

        for i in range(1, 10):
            qr_code_page_view(request)
            self.test_qrcode_page.refresh_from_db()
            self.assertEqual(self.test_qrcode_page.qr_code_usage, i)



from wagtail.admin.panels import (
    FieldPanel,
    MultiFieldPanel,
    ObjectList,
    TabbedInterface,
)
from wagtail.models import Page

from wagtail_qrcode.admin_forms import QrCodeEmailForm
from wagtail_qrcode.models import QRCodeMixin


class QRCodePage(QRCodeMixin, Page):
    qrcode_panels = QRCodeMixin.panels + [
        MultiFieldPanel(
            [
                FieldPanel("email_address"),
                FieldPanel("email_subject"),
                FieldPanel("email_body"),
            ],
            heading="Send QR code via email",
        )
    ]

    edit_handler = TabbedInterface(
        [
            ObjectList(Page.content_panels, heading="Content"),
            ObjectList(Page.promote_panels, heading="Promote"),
            ObjectList(Page.settings_panels, heading="Settings", classname="settings"),
            ObjectList(qrcode_panels, heading="QR Code", classname="qr-code"),
        ]
    )

    base_form_class = QrCodeEmailForm



from django import forms
from django.db import models
from wagtail.admin.panels import FieldPanel
from wagtail.documents import get_document_model
from wagtail.images import get_image_model

from wagtail_qrcode.field_panels import QrCodeFieldPanel, QrCodeUsageFieldPanel

ImageModel = get_image_model()
DocumentModel = get_document_model()


class QRCodeMixin(models.Model):
    GENERATE_QR_CODE = True

    qr_code_svg = models.TextField(
        verbose_name="QR Code SVG",
        blank=True,
        null=True,
    )

    qr_code_eps = models.ForeignKey(
        get_document_model(),
        verbose_name="QR Code EPS",
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )

    qr_code_usage = models.IntegerField(
        verbose_name="QR Code Usage Count",
        default=0,
    )

    class Meta:
        abstract = True

    panels = [
        QrCodeFieldPanel(
            "qr_code_svg",
            widget=forms.HiddenInput(),
        ),
        QrCodeUsageFieldPanel(
            "qr_code_usage",
            widget=forms.HiddenInput(),
        ),
        FieldPanel("qr_code_eps"),
    ]



from django.conf import settings
from django.core import mail
from django.test import RequestFactory, TestCase
from wagtail.documents import get_document_model
from wagtail.models import Page

from tests.testapp.models import QRCodePage
from wagtail_qrcode.wagtail_hooks import (
    delete_document,
    generate_qr_code,
    send_qr_code_email,
)


class TestWagtailHooks(TestCase):
    def test_generate_qr_code(self):
        request = RequestFactory().get("/")
        page = QRCodePage(title="Test Page")

        self.assertEqual(page.qr_code_svg, None)
        self.assertEqual(page.qr_code_eps, None)
        self.assertEqual(page.qr_code_usage, 0)

        root_page = Page.objects.get(id=1)
        home_page = root_page.get_children().first()
        home_page.add_child(instance=page)

        rev = page.save_revision()
        rev.publish()

        test_page = QRCodePage.objects.get(id=page.id)

        generate_qr_code(request, test_page)

        self.assertEqual(test_page.qr_code_svg[:4], "<svg")
        self.assertEqual(test_page.qr_code_eps.id, 1)
        self.assertEqual(test_page.qr_code_usage, 0)

        rev = test_page.save_revision()
        rev.publish()

        send_qr_code_email(
            test_page,
            email="foo@bar.com",
            subject="QR Code for Test Page",
            body="QR Code for Test Page",
        )

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].subject, "QR Code for Test Page")
        self.assertEqual(mail.outbox[0].body, "QR Code for Test Page")
        self.assertEqual(mail.outbox[0].from_email, settings.DEFAULT_FROM_EMAIL)
        self.assertEqual(mail.outbox[0].to, ["foo@bar.com"])
        self.assertEqual(
            mail.outbox[0].attachments[0][0], "qr-code-{}.eps".format(test_page.id)
        )

        delete_document(request, test_page)
        self.assertEqual(get_document_model().objects.count(), 0)



from django import forms
from wagtail.admin.forms import WagtailAdminPageForm

from wagtail_qrcode.wagtail_hooks import send_qr_code_email


class QrCodeEmailForm(WagtailAdminPageForm):
    email_address = forms.EmailField(
        label="Recipient Email",
        required=False,
        widget=forms.EmailInput(attrs={"placeholder": "Email address"}),
        help_text="The email address will not be saved to the database and is used only once to send the EPS file.",
    )
    email_subject = forms.CharField(
        label="Subject",
        required=False,
        widget=forms.TextInput(attrs={"placeholder": "Subject if email is sent"}),
    )
    email_body = forms.CharField(
        label="Body",
        required=False,
        widget=forms.Textarea(attrs={"placeholder": "Body of the email if sent"}),
    )

    def clean(self):
        cleaned_data = super().clean()

        if cleaned_data["email_address"]:
            if not cleaned_data["email_subject"]:
                self.add_error(
                    "email_subject",
                    "The email subject is required if an email address is provided.",
                )
            if not cleaned_data["email_body"]:
                self.add_error(
                    "email_body",
                    "The email body is required if an email address is provided.",
                )

        return cleaned_data

    def save(self, commit=True):
        page = super().save(commit=False)

        # send email if email_address is available
        if self.cleaned_data["email_address"]:
            email_address = self.cleaned_data["email_address"]
            email_subject = self.cleaned_data["email_subject"]
            email_body = self.cleaned_data["email_body"]
            send_qr_code_email(page, email_address, email_subject, email_body)

        if commit:
            page.save()
        return page



from django.test import TestCase

from wagtail_qrcode.apps import WagtailQrcodeAppConfig


class TestApps(TestCase):
    def test_config(self):
        self.assertEqual(WagtailQrcodeAppConfig.name, "wagtail_qrcode")
        self.assertEqual(WagtailQrcodeAppConfig.verbose_name, "Wagtail qrcode")
        self.assertEqual(WagtailQrcodeAppConfig.label, "wagtail_qrcode")



import io

import pyqrcode
from django.conf import settings
from django.core.files.base import File
from wagtail.documents import get_document_model
from wagtail.images import get_image_model
from wagtail.models import Collection

Document = get_document_model()
Image = get_image_model()


class WagtailQrCode:
    def __init__(self, page, collection=None):
        self.page = page
        # self.content = content
        self.collection = collection

        self.settings = self.settings()

    def settings(self):
        if hasattr(settings, "WAGTAIL_QR_CODE"):
            return settings.WAGTAIL_QR_CODE
        else:
            return {
                "collection_name": "QR Codes",
                "scale": 3,
                "quiet_zone": 6,
                "svg_has_xml_declaration": False,
                "svg_has_doc_type_declaration": False,
            }

    def build(self):
        title = self.page.title

        try:
            base_url = settings.WAGTAIL_QR_CODE_BASE_URL
        except AttributeError:
            base_url = settings.WAGTAILADMIN_BASE_URL

        # QR Code instance
        qrc = self.make_qr_code(f"{base_url}/qr-code?id={self.page.id}")

        # SVG QR code
        svg = self.make_svg(qrc)

        # EPS QR code and save a document
        eps = self.make_eps(qrc)
        document = self.make_document(title, eps, self.collection)

        return svg, document

    def make_qr_code(self, content):
        qrc = pyqrcode.create(content)
        return qrc

    def make_svg(self, qrc):
        svg_io = io.BytesIO()
        qrc.svg(
            svg_io,
            scale=self.settings["scale"],
            quiet_zone=self.settings["quiet_zone"],
            xmldecl=self.settings["svg_has_xml_declaration"],
            svgns=self.settings["svg_has_doc_type_declaration"],
            title=self.page.title,
        )
        svg_io = svg_io.getvalue().decode()

        return svg_io

    def make_eps(self, qrc):
        eps_io = io.StringIO()
        qrc.eps(
            eps_io,
            scale=self.settings["scale"],
            quiet_zone=self.settings["quiet_zone"],
        )

        return eps_io

    def make_document(self, title, eps, collection):
        doc = Document.objects.filter(title=title).first()

        if doc:
            # remove the document to create a replacement
            doc.delete()

        doc = Document(
            title=title,
            file=File(eps, name=f"qr-code-{self.page.id}.eps"),
            collection=collection,
        )
        doc.save()

        return doc


def create_collection(name):
    """Create a collection for the QR codes."""

    try:
        collection = Collection.objects.get(name=name)
    except Collection.DoesNotExist:
        root_collection = Collection.get_first_root_node()
        collection = root_collection.add_child(name=name)

    return collection



from django import forms
from django.test import TestCase

from wagtail_qrcode.field_panels import QrCodeFieldPanel, QrCodeUsageFieldPanel


class TestFieldPanels(TestCase):
    def test_qr_code_svg_field_panel(self):
        fp = QrCodeFieldPanel("foo", widget=forms.TextInput, classname="full")
        fp.field_name = "foo"

    def test_qr_code_svg_usage_field_panel(self):
        fp = QrCodeUsageFieldPanel("bar", widget=forms.TextInput, classname="full")
        fp.field_name = "bar"



"""
Django settings for temp project.

For more information on this file, see
https://docs.djangoproject.com/en/stable/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/stable/ref/settings/
"""

import os

# Build paths inside the project like this: os.path.join(PROJECT_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR = os.path.dirname(PROJECT_DIR)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/stable/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "a-not-so-secret-key"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]

EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"


# Application definition

INSTALLED_APPS = [
    "wagtail_qrcode",
    "tests",
    "tests.testapp",
    "wagtail.contrib.search_promotions",
    "wagtail.contrib.forms",
    "wagtail.contrib.redirects",
    "wagtail.embeds",
    "wagtail.users",
    "wagtail.snippets",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail.api.v2",
    "wagtail.contrib.modeladmin",
    "wagtail.contrib.routable_page",
    "wagtail.contrib.styleguide",
    "wagtail.sites",
    "wagtail",
    "taggit",
    "rest_framework",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.sitemaps",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

ROOT_URLCONF = "tests.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

# don't use the intentionally slow default password hasher
PASSWORD_HASHERS = ("django.contrib.auth.hashers.MD5PasswordHasher",)


# Database
# https://docs.djangoproject.com/en/stable/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

if os.getenv("DATABASE") == "postgres":
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.postgresql",
            "NAME": os.getenv("DB_NAME", "postgres"),
            "USER": os.getenv("DB_USER", "postgres"),
            "PASSWORD": os.getenv("DB_PASSWORD", "postgres"),
            "HOST": os.getenv("DB_HOST", "localhost"),
            "PORT": os.getenv("DB_PORT", "5432"),
        }
    }

if os.getenv("DATABASE") == "mysql":
    DATABASES = {
        "default": {
            "ENGINE": "django.db.backends.mysql",
            "NAME": os.getenv("DB_NAME", "mysql"),
            "USER": os.getenv("DB_USER", "root"),
            "PASSWORD": os.getenv("DB_PASSWORD", "mysql"),
            "HOST": os.getenv("DB_HOST", "127.0.0.1"),
            "PORT": os.getenv("DB_PORT", "3306"),
        }
    }

# Password validation
# https://docs.djangoproject.com/en/stable/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/stable/howto/static-files/

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

STATICFILES_DIRS = [os.path.join(PROJECT_DIR, "static")]

STATIC_ROOT = os.path.join(BASE_DIR, "test-static")
STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "test-media")


# Wagtail settings

WAGTAIL_SITE_NAME = "Wagtail qrcode test site"

WAGTAILADMIN_BASE_URL = "http://localhost:8000"

try:
    from .local import *  # noqa F403 F401
except ImportError:
    pass



from wagtail.admin.panels import FieldPanel


class QrCodeFieldPanel(FieldPanel):
    class BoundPanel(FieldPanel.BoundPanel):
        template_name = "wagtail_qrcode/admin/field_panel_4.html"


class QrCodeUsageFieldPanel(FieldPanel):
    class BoundPanel(FieldPanel.BoundPanel):
        template_name = "wagtail_qrcode/admin/usage_field_panel_4.html"



from django.test import TestCase
from django.urls.resolvers import RoutePattern

from wagtail_qrcode.urls import urlpatterns


class TestUrls(TestCase):
    def test_url_patterns(self):
        path = urlpatterns[0]
        self.assertEqual(path.name, "qr_code_page_view")
        self.assertIsInstance(path.pattern, RoutePattern)
        self.assertEqual(path.callback.__name__, "qr_code_page_view")



# Generated by Django 4.1.2 on 2022-10-21 21:47

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("wagtaildocs", "0012_uploadeddocument"),
        ("wagtailcore", "0066_collection_management_permissions"),
    ]

    operations = [
        migrations.CreateModel(
            name="QRCodePage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "qr_code_svg",
                    models.TextField(blank=True, null=True, verbose_name="QR Code SVG"),
                ),
                (
                    "qr_code_usage",
                    models.IntegerField(default=0, verbose_name="QR Code Usage Count"),
                ),
                (
                    "qr_code_eps",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="wagtaildocs.document",
                        verbose_name="QR Code EPS",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("wagtailcore.page", models.Model),
        ),
    ]



from django.contrib import admin
from django.urls import include, path
from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.documents import urls as wagtaildocs_urls

from wagtail_qrcode.views import qr_code_page_view

urlpatterns = [
    path("django-admin/", admin.site.urls),
    path("admin/", include(wagtailadmin_urls)),
    path("documents/", include(wagtaildocs_urls)),
    path("qr-code/", qr_code_page_view, name="qr-code-view"),
    path("", include(wagtail_urls)),
]



import io
import tempfile

import pyqrcode
from django.core.files.base import File
from django.test import TestCase, override_settings
from wagtail.documents import get_document_model
from wagtail.models import Collection, Page

from tests.testapp.models import QRCodePage
from wagtail_qrcode.cls import WagtailQrCode, create_collection

Document = get_document_model()


class TestWagtailQrCode(TestCase):
    def setUp(self):
        home_page = Page.objects.get(id=2)
        self.test_page = home_page.add_child(
            instance=QRCodePage(
                title="Test Page",
            ),
        )
        rev = self.test_page.save_revision()
        rev.publish()

    def test_settings(self):
        instance = WagtailQrCode(self.test_page)
        self.assertIsInstance(instance.settings["scale"], int)
        self.assertEqual(instance.settings["scale"], 3)
        self.assertIsInstance(instance.settings["quiet_zone"], int)
        self.assertEqual(instance.settings["quiet_zone"], 6)
        self.assertIsInstance(instance.settings["svg_has_xml_declaration"], bool)
        self.assertEqual(instance.settings["svg_has_xml_declaration"], False)
        self.assertIsInstance(instance.settings["svg_has_doc_type_declaration"], bool)
        self.assertEqual(instance.settings["svg_has_doc_type_declaration"], False)

    @override_settings(
        WAGTAIL_QR_CODE={
            "scale": 1,
            "quiet_zone": 2,
            "svg_has_xml_declaration": True,
            "svg_has_doc_type_declaration": True,
        }
    )
    def test_override_settings(self):
        instance = WagtailQrCode(self.test_page)
        self.assertIsInstance(instance.settings["scale"], int)
        self.assertEqual(instance.settings["scale"], 1)
        self.assertIsInstance(instance.settings["quiet_zone"], int)
        self.assertEqual(instance.settings["quiet_zone"], 2)
        self.assertIsInstance(instance.settings["svg_has_xml_declaration"], bool)
        self.assertEqual(instance.settings["svg_has_xml_declaration"], True)
        self.assertIsInstance(instance.settings["svg_has_doc_type_declaration"], bool)
        self.assertEqual(instance.settings["svg_has_doc_type_declaration"], True)

    def test_create_collection(self):
        collection = create_collection("Foo Collection")
        self.assertIsInstance(collection, Collection)
        self.assertEqual(collection.name, "Foo Collection")

    def test_create_collection_exists(self):
        count = Collection.objects.count()  # includes root collection
        root_collection = Collection.get_first_root_node()
        root_collection.add_child(name="Custom Collection")
        create_collection("Custom Collection")
        self.assertEqual(Collection.objects.count(), count + 1)

    def test_instance_past_collection(self):
        root_collection = Collection.get_first_root_node()
        collection = root_collection.add_child(name="Custom Collection")
        instance = WagtailQrCode(self.test_page, collection=collection)
        self.assertEqual(instance.collection, collection)

    def test_make_qr_code(self):
        instance = WagtailQrCode(self.test_page)
        qrc = instance.make_qr_code("foo string")
        self.assertIsInstance(qrc, pyqrcode.QRCode)
        self.assertEqual(str(qrc.data), "b'foo string'")

    def test_make_svg(self):
        instance = WagtailQrCode(self.test_page)
        qrc = pyqrcode.create("bar string")
        svg = instance.make_svg(qrc)
        self.assertIsInstance(svg, str)
        self.assertIn(
            '<svg height="111" width="111" class="pyqrcode"><title>Test Page</title>',
            svg,
        )
        self.assertEqual(svg[:4], "<svg")
        self.assertEqual(svg[-7:].replace("\n", ""), "</svg>")

    def test_make_eps(self):
        instance = WagtailQrCode(self.test_page)
        qrc = pyqrcode.create("baz string")
        eps = instance.make_eps(qrc)
        self.assertIsInstance(eps, io.StringIO)
        self.assertIsNotNone(eps.getvalue())

    def test_make_document(self):
        root_collection = Collection.get_first_root_node()
        collection = root_collection.add_child(name="Custom Collection")
        instance = WagtailQrCode(self.test_page, collection=collection)
        qrc = pyqrcode.create("foo string")
        eps = io.StringIO()
        qrc.eps(eps, scale=3, quiet_zone=3)
        document = instance.make_document("Foo Doc", eps, collection)
        self.assertIsInstance(document, Document)
        self.assertEqual(document.title, "Foo Doc")
        self.assertEqual(document.collection, collection)

    def test_make_document_exists(self):
        root_collection = Collection.get_first_root_node()
        collection = root_collection.add_child(name="Custom Collection")
        with tempfile.TemporaryFile() as f:
            f.write(b"content")
            doc = Document(
                title="Foo Doc",
                file=File(f, name="foo-doc.eps"),
            )
            doc.save()
        self.assertEqual(Document.objects.count(), 1)
        instance = WagtailQrCode(self.test_page, collection=collection)
        qrc = pyqrcode.create("foo string")
        eps = io.StringIO()
        qrc.eps(eps, scale=3, quiet_zone=3)
        instance.make_document("Foo Doc", eps, collection)
        self.assertEqual(Document.objects.count(), 1)

    def test_make_document_not_exists(self):
        root_collection = Collection.get_first_root_node()
        collection = root_collection.add_child(name="Custom Collection")
        with tempfile.TemporaryFile() as f:
            f.write(b"content")
            doc = Document(
                title="Foo Doc",
                file=File(f, name="foo-doc.eps"),
                collection=collection,
            )
            doc.save()
        self.assertEqual(Document.objects.count(), 1)

        instance = WagtailQrCode(self.test_page, collection=collection)
        qrc = pyqrcode.create("foo string")
        eps = io.StringIO()
        qrc.eps(eps, scale=3, quiet_zone=3)
        instance.make_document("Bar Doc", eps, collection)
        self.assertEqual(Document.objects.count(), 2)

    def test_build(self):
        collection = create_collection("Foo Collection")
        instance = WagtailQrCode(self.test_page, collection)
        svg, document = instance.build()
        self.assertIsInstance(svg, str)
        self.assertIsInstance(document, Document)



#!/usr/bin/env python

from os import path

from setuptools import find_packages, setup

from wagtail_qrcode import __version__

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="wagtail-qrcode",
    version=__version__,
    description="Create a QR code that can be used to link to a wagtail page",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Nick Moreton",
    author_email="nickmoreton@me.com",
    url="https://github.com/nickmoreton/wagtail-qrcode",
    packages=find_packages(),
    include_package_data=True,
    license="MIT",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Framework :: Django",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Wagtail",
        "Framework :: Wagtail :: 2",
        "Framework :: Wagtail :: 3",
        "Framework :: Wagtail :: 4",
    ],
    install_requires=["Wagtail>=4.1", "PyQRCode>=1,<2"],
    extras_require={
        "testing": ["coverage"],
    },
    zip_safe=False,
)



from django.conf import settings
from django.core.mail import EmailMessage
from wagtail import hooks
from wagtail.documents import get_document_model

from wagtail_qrcode.cls import WagtailQrCode, create_collection


@hooks.register("after_create_page")
@hooks.register("after_edit_page")
def generate_qr_code(request, page):
    """Add a QR code to the page."""

    # dont try to generate a qrcode if the page doesn't have all these fields
    if (
        hasattr(page, "qr_code_svg")
        and hasattr(page, "qr_code_eps")
        and hasattr(page, "qr_code_usage")
    ):
        collection = create_collection("QR Codes")

        qrc = WagtailQrCode(page, collection)

        svg, document = qrc.build()

        page.qr_code_svg = svg
        page.qr_code_eps = document

        rev = page.save_revision()
        rev.publish()


def send_qr_code_email(page, email=None, subject=None, body=None):
    """Send the QR code to the email address."""
    doc = page.qr_code_eps
    if doc is None or email is None:
        return

    if subject is None:
        subject = f"QR Code for {page.title}"
    if body is None:
        body = f"QR Code for {page.title}"

    email = EmailMessage(
        subject=subject,
        body=body,
        from_email=settings.DEFAULT_FROM_EMAIL,
        to=[email],
        attachments=[("qr-code-{}.eps".format(page.id), doc.file.read(), "image/eps")],
    )
    try:
        email.send()
    except Exception as e:
        print(e)


@hooks.register("after_delete_page")
def delete_document(request, page):
    doc = get_document_model().objects.filter(id=page.qr_code_eps.id)
    if doc:
        doc.delete()



from django.test import TestCase

from tests.testapp.models import QRCodePage


class TestModels(TestCase):
    def test_qrcode_mixin(self):
        test_page = QRCodePage()

        self.assertTrue(hasattr(test_page, "GENERATE_QR_CODE"))
        self.assertTrue(test_page.GENERATE_QR_CODE, True)

        self.assertTrue(hasattr(test_page, "qr_code_svg"))
        self.assertTrue(hasattr(test_page, "qr_code_eps"))
        self.assertTrue(hasattr(test_page, "qr_code_usage"))

        panels = test_page.panels

        self.assertTrue(len(panels) >= 3)



from django.http import HttpResponseNotFound, HttpResponseRedirect
from wagtail.models import Page


def qr_code_page_view(request):
    """QR code redirect view."""
    page_id = int(request.GET.get("id"))

    try:
        specific_cls = Page.objects.get(id=page_id).specific_class
        page = specific_cls.objects.get(id=page_id)
    except Page.DoesNotExist:
        return HttpResponseNotFound("Page not found")

    if hasattr(page, "qr_code_usage"):
        page.qr_code_usage += 1
        page.save()
        return HttpResponseRedirect(page.url)

    return HttpResponseNotFound("Page not found")
